#!/usr/bin/env python3
#
# SPDX-FileCopyrightText: 2025 Carlos M. Fonseca <cmfonsec@dei.uc.pt>
#
# SPDX-License-Identifier: Apache-2.0


from api.base import Neighbourhood, Move


# ------------------------------- Neighbourhood ------------------------------

class AddNeighbourhood(Neighbourhood):
    def moves(self, solution):
        assert self.problem == solution.problem
        i = solution.tour[-1]
        for j in solution.not_visited:
            yield AddMove(self, i, j)



# ----------------------------------- Move -----------------------------------

class AddMove(Move):
    def __init__(self, neighbourhood, i, j):
        self.neighbourhood = neighbourhood
        # i and j are cities
        self.i = i
        self.j = j

    def __str__(self):
        return "%d %d" % (self.i, self.j)

    def apply(self, solution):
        assert solution.tour[-1] == self.i
        prob = solution.problem
        # Update lower bound
        solution.lb += prob.dist[self.i][self.j]
        if len(solution.not_visited) == 1:
            solution.lb += prob.dist[self.j][solution.tour[0]]
        # Tighter, but *not* better!
        # solution.lb += prob.dist[self.j][solution.tour[0]] - prob.dist[self.i][solution.tour[0]]
        # Update solution
        solution.tour.append(self.j)
        solution.not_visited.remove(self.j)

    def lower_bound_increment(self, solution):
        assert solution.tour[-1] == self.i
        prob = solution.problem
        incr = prob.dist[self.i][self.j]
        if len(solution.not_visited) == 1:
            incr += prob.dist[self.j][solution.tour[0]]
        # Tighter, but *not* better!
        # incr += prob.dist[self.j][solution.tour[0]] - prob.dist[self.i][solution.tour[0]]
        return incr