#!/usr/bin/env python3
#
# SPDX-FileCopyrightText: 2025 Carlos M. Fonseca <cmfonsec@dei.uc.pt>
#
# SPDX-License-Identifier: Apache-2.0

import math
import random

class Problem:
    def __init__(self, xy):
        self.n = len(xy)
        self.xy = tuple([(x,y) for x,y in xy])
        dist = []
        for i in range(self.n):
            aux = []
            for j in range(self.n):
                aux.append(math.sqrt((self.xy[i][0]-self.xy[j][0])**2 + (self.xy[i][1]-self.xy[j][1])**2))
            dist.append(tuple(aux))
        self.dist = tuple(dist)

    def __str__(self):
        out = [str(self.n)]
        for i, j in self.xy:
            out.append("%d %d" % (i, j))
        return "\n".join(out)

    @classmethod
    def from_textio(cls, f):
        """
        Create a problem from a text I/O source `f`
        """
        n = int(f.readline())
        xy = []
        for i in range(n):
            xy.append(tuple(map(float, f.readline().split())))
        return cls(xy)

    @classmethod
    def random_cities(cls, n, seed=1234, width=1000, height=1000):
        """
        Create a problem generating the cities at random in a width x heigh rectangle
        """
        random.seed(seed) 
        xy = []
        for i in range(n):
            xy.append( (random.randrange(width), random.randrange(height)) )
        return cls(xy)
    


