#!/usr/bin/env python3
#
# SPDX-FileCopyrightText: 2025 Carlos M. Fonseca <cmfonsec@dei.uc.pt>
#
# SPDX-License-Identifier: Apache-2.0

import roas.simple_solvers as solvers
from api.problem import Problem
from api.solution import Solution
import sys
import matplotlib.pyplot as plt  

from log_config import setup_logging
setup_logging()
import logging
logger = logging.getLogger("Main")

if __name__ == '__main__':
    if len(sys.argv)==2:
        with open(sys.argv[1], 'r') as f:
            prob = Problem.from_textio(f)
        #prob = tsp.Problem.from_textio(sys.stdin)
    else:
        prob = Problem.random_cities(12, seed=43, width=1000, height=1000)
    if (prob):
        # sol = solvers.greedy_construction(Solution.empty_solution(prob))
        # print("Greedy construction: {:.2f}".format(sol.objective_value()))
        # sol.plot_tour()
        # plt.show()
        sol1 = solvers.greedy_construction_random_tie_breaking(Solution.empty_solution(prob))
        print("Greedy construction with random tie breaking: {:.2f}".format(sol1.objective_value()))
        # sol1.plot_tour()
        # plt.show()
        # sol2 = Solution.random_solution(prob)
        # print("Random solution: {:.2f}".format(sol2.objective_value()))
        # print("Neighbourhood size:", len(list(sol2.local_neighbourhood().moves(sol2))))
        # sol3 = solvers.first_improvement(sol2.copy())
        # print("First improvement: {:.2f}".format(sol3.objective_value()))
        # sol4 = solvers.best_improvement(sol2.copy())
        # print("Best improvement: {:.2f}".format(sol4.objective_value()))
        
        #print("Starting at heuristic solution (gcwrtb)")
        sol5 = solvers.first_improvement(sol1.copy())
        print("First improvement: {:.2f}".format(sol5.objective_value()))
        sol6 = solvers.best_improvement(sol5.copy())
        print("Best improvement: {:.2f}".format(sol6.objective_value()))
        #sol5.plot_tour()
        #plt.show()




